#version 330

uniform sampler2D tex;
uniform mat4 model;

struct Flashlight {
	vec3 position;
	float constAttenuation;
	float linearAttenuation;
	float quadraticAttenuation;
	vec3 coneDirection;
	float coneInnerAngle;
	float coneOuterAngle;
};

uniform Flashlight flashlight;

in vec3 fragPosition;
in vec3 fragNormal;
in vec2 fragUV;

out vec4 fragColor;

void main()
{	
	// Calculate normal in world coordinates
    mat3 normalMatrix = transpose(inverse(mat3(model)));
    vec3 normal = normalize(normalMatrix * fragNormal);

    // Calculate the location of this fragment (pixel) in world coordinates
    vec3 position = vec3(model * vec4(fragPosition, 1));
	
	// Get texture color
	vec4 surfaceColor = vec4(texture(tex, fragUV).rgb, 1);
	if(vec3(surfaceColor) == vec3(0, 0, 0))
	{
		surfaceColor.a = 0;
	}
	
	// Calculate the vector from this pixels surface to the light source
	vec3 surfaceToLight = flashlight.position - position;

	float lightToSurfaceAngle = dot(-normalize(surfaceToLight), normalize(flashlight.coneDirection));

	if(lightToSurfaceAngle > flashlight.coneOuterAngle) { // in light cone

		// Calculate the cosine of the angle of incidence
		float brightness = dot(normal, surfaceToLight) / (length(surfaceToLight) * length(normal));
		brightness = clamp(brightness, 0, 1);

		// Calculate attenuation
		float d = length(surfaceToLight);
		float attenuation = flashlight.constAttenuation + d * flashlight.linearAttenuation + (d * d) * flashlight.quadraticAttenuation;
					
		// Calculate intensity of pixel for current light
		vec4 intensity = (vec4(1) / attenuation) * brightness;
		float alpha = intensity.a;

		if(lightToSurfaceAngle <= flashlight.coneInnerAngle) {
			// Lerp between inner and outer cone
			//intensity = intensity * ((lightToSurfaceAngle - flashlight.coneOuterAngle) / (flashlight.coneInnerAngle - flashlight.coneOuterAngle));
			alpha = alpha * ((lightToSurfaceAngle - flashlight.coneOuterAngle) / (flashlight.coneInnerAngle - flashlight.coneOuterAngle));
		}
	
		fragColor = vec4(intensity.rgb, alpha) * surfaceColor;
	} else {
		fragColor = vec4(0, 0, 0, 0);
	}
}