#version 330

#define MAX_LIGHTS 10

uniform mat4 model;
uniform mat4 viewMatrix;
uniform sampler2D tex;
//uniform int useTexture;
uniform int numLights;
uniform vec3 ambientLight;
uniform float specularFactor;
uniform float specularAlpha;
uniform vec3 cameraPosition;
uniform vec3 fogColor;
uniform bool fogEnabled;

in vec4 worldPosition;
in vec3 worldNormal;
in vec4 outColor;
in vec2 outTexCoord;

out vec4 fragColor;
	
uniform struct Light {
	int type;
	bool enabled;
	vec3 intensity;
	vec3 position;
	float constAttenuation;
	float linearAttenuation;
	float quadraticAttenuation;
	vec3 coneDirection;
	float coneInnerAngle;
	float coneOuterAngle;
} lights[MAX_LIGHTS];


vec3 getDiffuseLightIntensity(vec3 position, vec3 normal, Light light);
float getFogFactor();

void main()
{
	// Calculate normal in world coordinates
    vec3 normal = normalize(worldNormal);
    
    // Calculate the location of this fragment (pixel) in world coordinates
    vec3 position = worldPosition.xyz-worldPosition.w;
	
	// Get texture color
	vec4 surfaceColor = vec4(texture(tex, outTexCoord).rgb, 1);

	vec3 allIntensities = ambientLight;
	vec3 surfaceToLight = vec3(0);
	float brightness = 0;
    
	for(int i = 0; i < numLights; i++) 
	{
		if(!lights[i].enabled) continue;

		allIntensities += getDiffuseLightIntensity(position, normal, lights[i]);
	}	

	// Calculate final color dependend on texture color and all lights
    fragColor = vec4(allIntensities * surfaceColor.rgb, surfaceColor.a);

	// Mix with fog (LERP)
	float fogFactor = 1.0f;
	if(fogEnabled) fogFactor = getFogFactor();
	fragColor = vec4(fogColor, 1) * (1 - fogFactor) + fragColor * fogFactor;
}

float getFogFactor()
{
	// Linear fog
	vec4 viewPos = viewMatrix * worldPosition;
	float dist = min(60.0f, abs(viewPos.z));
	float fogFactor = 1.0f / exp(pow(dist * 0.02f, 2));
	fogFactor = clamp(fogFactor, 0.0f, 1.0f);

	return fogFactor;
}

vec3 getDiffuseLightIntensity(vec3 position, vec3 normal, Light light) 
{
	vec3 surfaceToLight = vec3(0);
	float brightness = 0;
	float attenuation = 0;

	switch(light.type) 
	{
		case 0: // POINT LIGHTS
			// Calculate the vector from this pixels surface to the light source
			surfaceToLight = light.position - position;

			// Calculate the cosine of the angle of incidence
			brightness = dot(normal, surfaceToLight) / (length(surfaceToLight) * length(normal));
			brightness = clamp(brightness, 0, 1);

			// Calculate attenuation
			float d = length(surfaceToLight);
			attenuation = light.constAttenuation + d * light.linearAttenuation + (d * d) * light.quadraticAttenuation;

			// Calculate intensity of pixel for current light
			return (light.intensity / attenuation) * brightness;

		case 1: // DIRECTIONAL LIGHTS
			// Calculate the vector from this pixels surface to the light source
			surfaceToLight = normalize(light.position) * -1;

			// Calculate the cosine of the angle of incidence
			brightness = dot(normal, surfaceToLight) / (length(surfaceToLight) * length(normal));
			brightness = clamp(brightness, 0, 1);
				
			// Calculate intensity of pixel for current light
			return light.intensity * brightness;

			break;

		case 2: // SPOT LIGHTS        
			// Calculate the vector from this pixels surface to the light source
			surfaceToLight = light.position - position;

			float lightToSurfaceAngle = dot(-normalize(surfaceToLight), normalize(light.coneDirection));

			if(lightToSurfaceAngle > light.coneOuterAngle) { // in light cone

				// Calculate the cosine of the angle of incidence
				brightness = dot(normal, surfaceToLight) / (length(surfaceToLight) * length(normal));
				brightness = clamp(brightness, 0, 1);

				// Calculate attenuation
				float d = length(surfaceToLight);
				attenuation = light.constAttenuation + d * light.linearAttenuation + (d * d) * light.quadraticAttenuation;
					
				// Calculate intensity of pixel for current light
				vec3 intensity = (light.intensity / attenuation) * brightness;

				if(lightToSurfaceAngle <= light.coneInnerAngle) {
					// Lerp between inner and outer cone
					intensity = intensity * ((lightToSurfaceAngle - light.coneOuterAngle) / (light.coneInnerAngle - light.coneOuterAngle));
				}

				return intensity;
			}

			return vec3(0);
	} 
}

/*#version 330

#define MAX_LIGHTS 10

uniform sampler2D tex;
uniform int useTexture;
uniform int numLights;
uniform vec3 ambientLight;

in vec4 worldPosition;
in vec3 worldNormal;
in vec4 outColor;
in vec2 outTexCoord;

out vec4 fragColor;
	
uniform struct Light {
	int type;
	bool enabled;
	vec3 intensity;
	vec3 position;
	float constAttenuation;
	float linearAttenuation;
	float quadraticAttenuation;
	vec3 coneDirection;
	float coneInnerAngle;
	float coneOuterAngle;
} lights[MAX_LIGHTS];


void main()
{
	// Calculate normal in world coordinates
    vec3 normal = normalize(worldNormal);
    
    // Calculate the location of this fragment (pixel) in world coordinates
    vec3 position = worldPosition.xyz-worldPosition.w;
	
	// Get texture color
	vec4 surfaceColor = vec4(texture(tex, outTexCoord).rgb, 1);

	vec3 allIntensities = ambientLight;
	vec3 surfaceToLight = vec3(0);
	float brightness = 0;
    
	for(int i = 0; i < numLights; i++) 
	{
		if(!lights[i].enabled) continue;

		switch(lights[i].type) 
		{
			case 0: // POINT LIGHTS
				// Calculate the vector from this pixels surface to the light source
				surfaceToLight = lights[i].position - position;

				// Calculate the cosine of the angle of incidence
				brightness = dot(normal, surfaceToLight) / (length(surfaceToLight) * length(normal));
				brightness = clamp(brightness, 0, 1);

				// Calculate attenuation
				float d = length(surfaceToLight);
				float attenuation = lights[i].constAttenuation + d * lights[i].linearAttenuation + (d * d) * lights[i].quadraticAttenuation;

				// Calculate intensity of pixel for current light
				allIntensities += (lights[i].intensity / attenuation) * brightness;

				break;

			case 1: // DIRECTIONAL LIGHTS
				// Calculate the vector from this pixels surface to the light source
				surfaceToLight = normalize(lights[i].position) * -1;

				// Calculate the cosine of the angle of incidence
				brightness = dot(normal, surfaceToLight) / (length(surfaceToLight) * length(normal));
				brightness = clamp(brightness, 0, 1);
				
				// Calculate intensity of pixel for current light
				allIntensities += lights[i].intensity * brightness;

				break;

			case 2: // SPOT LIGHTS        
				// Calculate the vector from this pixels surface to the light source
				surfaceToLight = lights[i].position - position;

				attenuation = 0.0f;
				float lightToSurfaceAngle = dot(-normalize(surfaceToLight), normalize(lights[i].coneDirection));

				if(lightToSurfaceAngle > lights[i].coneOuterAngle) { // in light cone

					// Calculate the cosine of the angle of incidence
					brightness = dot(normal, surfaceToLight) / (length(surfaceToLight) * length(normal));
					brightness = clamp(brightness, 0, 1);

					// Calculate attenuation
					float d = length(surfaceToLight);
					attenuation = lights[i].constAttenuation + d * lights[i].linearAttenuation + (d * d) * lights[i].quadraticAttenuation;
					
					// Calculate intensity of pixel for current light
					vec3 intensity = (lights[i].intensity / attenuation) * brightness;

					if(lightToSurfaceAngle <= lights[i].coneInnerAngle) {
						// Lerp between inner and outer cone
						intensity = intensity * ((lightToSurfaceAngle - lights[i].coneOuterAngle) / (lights[i].coneInnerAngle - lights[i].coneOuterAngle));
					}
					allIntensities += intensity;
				}

				break;
		} 
	}

	// Calculate final color dependend on texture color and all lights
    fragColor = vec4(allIntensities * surfaceColor.rgb, surfaceColor.a);
}*/

